############################# PSQF 6272 Example 2 using R #####################

# Set width of output and number of significant digits printed,
# number of digits before using scientific notation, shut off significance stars
options(width=120, digits=8, scipen=9, show.signif.stars=FALSE)

#####  Check to see if packages are downloaded, install if not, then load  #####

if (!require("readxl")) install.packages("readxl")
library(readxl) # To import xls or xlsx data as table

if (!require("expss")) install.packages("expss")
library(expss) # To add variable and value labels, sort

if (!require("psych")) install.packages("psych")
library(psych) # To add summary functions

if (!require("lme4")) install.packages("lme4")
library(lme4) # To estimate MLMs using lmer

if (!require("lmerTest")) install.packages("lmerTest")
library(lmerTest) # To get Satterthwaite DDF in lmer

if (!require("performance")) install.packages("performance")
library(performance) # To get ICC using lmer

if (!require("nlme")) install.packages("nlme")
library(nlme) # To estimate MLMs using lme or gls

if (!require("psychometric")) install.packages("psychometric")
library(psychometric) # To ICCs using lme or gls

if (!require("r2mlm")) install.packages("r2mlm")
library(r2mlm) # To compute pseudo-R2 from one model

if (!require("TeachingDemos")) install.packages("TeachingDemos")
library(TeachingDemos) # To create text output files

# Clear workspace (re-run as needed for troubleshooting purposes)
#rm(list = ls())

################################################################################
#####          BEGIN DATA IMPORT AND PREPARATION FOR EXAMPLE 2             #####
#####              CHANGE "filesave" TO YOUR FOLDER PATH                   #####
################################################################################

# Define variables for working directory and data name -- CHANGE THESE
filesave = "C:\\Dropbox/23_PSQF6272/PSQF6272_Example2/"
filename = "Example2_Data.xlsx"
setwd(dir=filesave)

# Load Jonathan's custom R functions from folder within working directory
functions = paste0("R functions/",dir("R functions/"))
temp = lapply(X=functions, FUN=source)

#  Import Example 2 excel data file in sheet "HSB4"
Example2 = read_excel(paste0(filesave,filename), sheet="HSB4") 
# Convert to data frame to use in analysis
Example2 = as.data.frame(Example2)

print("R Descriptive Statistics for Example 2 Variables")
describe(x=Example2[ , c("math","size","private")])

# Center and re-scale school size
Example2$size100 = (Example2$size-1000)/100
# private is already 0/1

# Filter to only cases complete on all variables to be used below
Example2 = Example2[complete.cases(Example2[ , c("math","size","private")]),]

# Open external file to save results to (turn off to use console instead)
#txtStart(file=paste0(filesave,"Example2_R_Output.txt"))


################################################################################
#####                 BEGIN MODELS FOR EXAMPLE 2                           #####
################################################################################

print("R Model 0: Single-Level Empty Means, NO Random Intercept")
Model0 = gls(data=Example2, model=math~1)
print("Show results including -2LL and residual variance")
-2*logLik(Model0); summary(Model0); summary(Model0)$sigma^2

txtComment(" ") # insert blank space in output

################################################################################
print("R Model 1: Two-Level Empty Means, WITH Random Intercept")
Model1 = lmer(data=Example2, REML=TRUE, formula=math~1+(1|schoolID))
print("Show results using Satterthwaite DDF including -2LL as deviance")
llikAIC(Model1, chkREML=FALSE); summary(Model1, ddf="Satterthwaite")

print("Show intraclass correlation and its LRT")
icc(Model1); ranova(Model1)
print("Show ICC2 for Reliability of School Mean Math -- Weighted and Unweighted")
ICC2.lme(data=Example2, dv=math, grp=schoolID, weighted=TRUE)
ICC2.lme(data=Example2, dv=math, grp=schoolID, weighted=FALSE)

txtComment(" ") # insert blank space in output

##################################################################################
print("R Model 2: Add Main Effects of Level-2 School Size and Public vs Private")
Model2 = lmer(data=Example2, REML=TRUE, formula=math~1+size100+private+(1|schoolID))
print("Show results using Satterthwaite DDF including -2LL as deviance")
llikAIC(Model2, chkREML=FALSE); summary(Model2, ddf="Satterthwaite")

print("F-Test of Model Total-R2")
contestMD(Model2, ddf="Satterthwaite", L=rbind(c(0,1,0),c(0,0,1)))

print("Psuedo-R2 relative to empty model using Jonathan's function")
pseudoRSquaredinator(smallerModel=Model1, largerModel=Model2)

print("Rights & Sterba R2 suite -- f2 is for random intercept")
r2mlm(model=Model2, bargraph=FALSE)

print("Total-R2 relative to empty model using Jonathan's function")
totalRSquaredinator(model=Model2, dvName="math", data=Example2)

# Compute total-R2 the longer way instead by saving predicted outcomes
Example2$PredMain = predict(Model2, re.form=NA) # Do not include random intercept
rModel2 = cor.test(Example2$PredMain, Example2$math, method="pearson")
print("Total R2"); rModel2$estimate^2

txtComment(" ") # insert blank space in output


######################################################################################
print("R Model 3: Add Interaction of Level-2 School Size and Public vs Private")
Model3 = lmer(data=Example2, REML=TRUE, formula=math~1+size100*private+(1|schoolID))
print("Show results using Satterthwaite DDF including -2LL as deviance")
llikAIC(Model3, chkREML=FALSE); summary(Model3, ddf="Satterthwaite")

print("Size slope for private schools?"); contest1D(Model3, L=c(0,1,0,1))
print("F-Test of Model Total-R2")
contestMD(Model3, ddf="Satterthwaite", L=rbind(c(0,1,0,0),c(0,0,1,0),c(0,0,0,1)))

print("Psuedo-R2 relative to empty model using Jonathan's function")
pseudoRSquaredinator(smallerModel=Model1, largerModel=Model3)

print("Rights & Sterba R2 suite -- f2 is for random intercept")
r2mlm(model=Model3, bargraph=FALSE)

print("Total-R2 relative to empty model using Jonathan's function")
totalRSquaredinator(model=Model3, dvName="math", data=Example2)

txtComment(" ") # insert blank space in output


#######################################################################################
# Close external file of saved results
#txtStop()



